@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -44,7 +44,9 @@ module ApplicationHelper
   # @param [optional, Hash] html_options Options passed to link_to
   # @param [optional, Hash] parameters_for_method_reference Extra parameters for link_to
   def link_to_if_authorized(name, options = {}, html_options = nil, *parameters_for_method_reference)
-    link_to(name, options, html_options, *parameters_for_method_reference) if authorize_for(options[:controller] || params[:controller], options[:action])
+    if authorize_for(options[:controller] || params[:controller], options[:action])
+      link_to(name, options, html_options, *parameters_for_method_reference)
+    end
   end
 
   # Displays a link to user's account page if active
@@ -135,11 +137,13 @@ module ApplicationHelper
     text = options.delete(:text) || format_revision(revision)
     rev = revision.respond_to?(:identifier) ? revision.identifier : revision
     link_to(
-        h(text),
-        {:controller => 'repositories', :action => 'revision', :id => repository.project, :repository_id => repository.identifier_param, :rev => rev},
-        :title => l(:label_revision_id, format_revision(revision)),
-        :accesskey => options[:accesskey]
-      )
+      h(text),
+      {:controller => 'repositories', :action => 'revision',
+       :id => repository.project,
+       :repository_id => repository.identifier_param, :rev => rev},
+      :title => l(:label_revision_id, format_revision(revision)),
+      :accesskey => options[:accesskey]
+    )
   end
 
   # Generates a link to a message
@@ -186,21 +190,28 @@ module ApplicationHelper
   # Generates a link to a version
   def link_to_version(version, options = {})
     return '' unless version && version.is_a?(Version)
+
     options = {:title => format_date(version.effective_date)}.merge(options)
     link_to_if version.visible?, format_version_name(version), version_path(version), options
   end
 
   RECORD_LINK = {
-    'CustomValue'  => -> (custom_value) { link_to_record(custom_value.customized) },
-    'Document'     => -> (document)     { link_to(document.title, document_path(document)) },
-    'Group'        => -> (group)        { link_to(group.name, group_path(group)) },
-    'Issue'        => -> (issue)        { link_to_issue(issue, :subject => false) },
-    'Message'      => -> (message)      { link_to_message(message) },
-    'News'         => -> (news)         { link_to(news.title, news_path(news)) },
-    'Project'      => -> (project)      { link_to_project(project) },
-    'User'         => -> (user)         { link_to_user(user) },
-    'Version'      => -> (version)      { link_to_version(version) },
-    'WikiPage'     => -> (wiki_page)    { link_to(wiki_page.pretty_title, project_wiki_page_path(wiki_page.project, wiki_page.title)) }
+    'CustomValue' => lambda {|custom_value| link_to_record(custom_value.customized)},
+    'Document'    => lambda {|document| link_to(document.title, document_path(document))},
+    'Group'       => lambda {|group|    link_to(group.name, group_path(group))},
+    'Issue'       => lambda {|issue|    link_to_issue(issue, :subject => false)},
+    'Message'     => lambda {|message|  link_to_message(message)},
+    'News'        => lambda {|news|     link_to(news.title, news_path(news))},
+    'Project'     => lambda {|project|  link_to_project(project)},
+    'User'        => lambda {|user|     link_to_user(user)},
+    'Version'     => lambda {|version|  link_to_version(version)},
+    'WikiPage'    =>
+      lambda do |wiki_page|
+        link_to(
+          wiki_page.pretty_title,
+          project_wiki_page_path(wiki_page.project, wiki_page.title)
+        )
+      end
   }
 
   def link_to_record(record)
@@ -212,8 +223,10 @@ module ApplicationHelper
   ATTACHMENT_CONTAINER_LINK = {
     # Custom list, since project/version attachments are listed in the files
     # view and not in the project/milestone view
-    'Project'      => -> (project)      { link_to(l(:project_module_files), project_files_path(project)) },
-    'Version'      => -> (version)      { link_to(l(:project_module_files), project_files_path(version.project)) },
+    'Project' =>
+       lambda {|project| link_to(l(:project_module_files), project_files_path(project))},
+    'Version' =>
+       lambda {|version| link_to(l(:project_module_files), project_files_path(version.project))},
   }
 
   def link_to_attachment_container(attachment_container)
@@ -253,7 +266,20 @@ module ApplicationHelper
     when 'Issue'
       object.visible? && html ? link_to_issue(object) : "##{object.id}"
     when 'Attachment'
-      html ? link_to_attachment(object) : object.filename
+      if html
+        content_tag(
+          :span,
+          link_to_attachment(object) +
+          link_to_attachment(
+            object,
+            :class => ['icon-only', 'icon-download'],
+            :title => l(:button_download),
+            :download => true
+          )
+        )
+      else
+        object.filename
+      end
     when 'CustomValue', 'CustomFieldValue'
       if object.custom_field
         f = object.custom_field.format.formatted_custom_value(self, object, html)
@@ -271,7 +297,8 @@ module ApplicationHelper
   end
 
   def wiki_page_path(page, options={})
-    url_for({:controller => 'wiki', :action => 'show', :project_id => page.project, :id => page.title}.merge(options))
+    url_for({:controller => 'wiki', :action => 'show', :project_id => page.project,
+             :id => page.title}.merge(options))
   end
 
   def thumbnail_tag(attachment)
@@ -343,8 +370,8 @@ module ApplicationHelper
   end
 
   def format_activity_description(text)
-    h(text.to_s.truncate(120).gsub(%r{[\r\n]*<(pre|code)>.*$}m, '...')
-       ).gsub(/[\r\n]+/, "<br />").html_safe
+    h(text.to_s.truncate(120).gsub(%r{[\r\n]*<(pre|code)>.*$}m, '...')).
+      gsub(/[\r\n]+/, "<br />").html_safe
   end
 
   def format_version_name(version)
@@ -362,7 +389,12 @@ module ApplicationHelper
 
   def due_date_distance_in_words(date)
     if date
-      l((date < User.current.today ? :label_roadmap_overdue : :label_roadmap_due_in), distance_of_date_in_words(User.current.today, date))
+      l((if date < User.current.today
+           :label_roadmap_overdue
+         else
+           :label_roadmap_due_in
+         end),
+        distance_of_date_in_words(User.current.today, date))
     end
   end
 
@@ -378,7 +410,7 @@ module ApplicationHelper
         # set the project environment to please macros.
         @project = project
         if ancestors.empty? || project.is_descendant_of?(ancestors.last)
-          s << "<ul class='projects #{ ancestors.empty? ? 'root' : nil}'>\n"
+          s << "<ul class='projects #{ancestors.empty? ? 'root' : nil}'>\n"
         else
           ancestors.pop
           s << "</li>"
@@ -388,6 +420,7 @@ module ApplicationHelper
           end
         end
         classes = (ancestors.empty? ? 'root' : 'child')
+        classes += ' archived' if project.archived?
         s << "<li class='#{classes}'><div class='#{classes}'>"
         s << h(block_given? ? capture(project, &block) : project.name)
         s << "</div>\n"
@@ -408,10 +441,19 @@ module ApplicationHelper
         if controller.controller_name == 'wiki' && controller.action_name == 'export'
           href = "##{page.title}"
         else
-          href = {:controller => 'wiki', :action => 'show', :project_id => page.project, :id => page.title, :version => nil}
+          href = {:controller => 'wiki', :action => 'show',
+                  :project_id => page.project, :id => page.title, :version => nil}
         end
-        content << link_to(h(page.pretty_title), href,
-                           :title => (options[:timestamp] && page.updated_on ? l(:label_updated_time, distance_of_time_in_words(Time.now, page.updated_on)) : nil))
+        content <<
+          link_to(
+            h(page.pretty_title),
+            href,
+            :title => (if options[:timestamp] && page.updated_on
+                         l(:label_updated_time, distance_of_time_in_words(Time.now, page.updated_on))
+                       else
+                         nil
+                       end)
+          )
         content << "\n" + render_page_hierarchy(pages, page.id, options) if pages[page.id]
         content << "</li>\n"
       end
@@ -423,7 +465,7 @@ module ApplicationHelper
   # Renders flash messages
   def render_flash_messages
     s = +''
-    flash.each do |k,v|
+    flash.each do |k, v|
       s << content_tag('div', v.html_safe, :class => "flash #{k}", :id => "flash_#{k}")
     end
     s.html_safe
@@ -470,28 +512,31 @@ module ApplicationHelper
     end
   end
 
-  def render_projects_for_jump_box(projects, selected=nil)
-    jump_box = Redmine::ProjectJumpBox.new User.current
-    query = params[:q] if request.format.js?
-    bookmarked = jump_box.bookmarked_projects(query)
-    recents = jump_box.recently_used_projects(query)
-    projects = projects - (recents + bookmarked)
-    projects_label = (bookmarked.any? || recents.any?) ? :label_optgroup_others : :label_project_plural
+  def render_projects_for_jump_box(projects, selected: nil, query: nil)
+    if query.blank?
+      jump_box = Redmine::ProjectJumpBox.new User.current
+      bookmarked = jump_box.bookmarked_projects
+      recents = jump_box.recently_used_projects
+      projects_label = :label_project_all
+    else
+      projects_label = :label_result_plural
+    end
     jump = params[:jump].presence || current_menu_item
     s = (+'').html_safe
-    build_project_link = ->(project, level = 0){
+    build_project_link = lambda do |project, level = 0|
       padding = level * 16
       text = content_tag('span', project.name, :style => "padding-left:#{padding}px;")
       s << link_to(text, project_path(project, :jump => jump),
                    :title => project.name,
                    :class => (project == selected ? 'selected' : nil))
-    }
+    end
     [
       [bookmarked, :label_optgroup_bookmarks, true],
       [recents,   :label_optgroup_recents,    false],
       [projects,  projects_label,             true]
     ].each do |projects, label, is_tree|
       next if projects.blank?
+
       s << content_tag(:strong, l(label))
       if is_tree
         project_tree(projects, &build_project_link)
@@ -522,11 +567,10 @@ module ApplicationHelper
     content =
       content_tag('div',
                   content_tag('div', q, :class => 'quick-search') +
-                    content_tag('div', render_projects_for_jump_box(projects, @project),
+                    content_tag('div', render_projects_for_jump_box(projects, selected: @project),
                                 :class => 'drdn-items projects selection') +
                     content_tag('div', all, :class => 'drdn-items all-projects selection'),
-                  :class => 'drdn-content'
-      )
+                  :class => 'drdn-content')
     content_tag('div', trigger + content, :id => "project-jump", :class => "drdn")
   end
 
@@ -541,7 +585,8 @@ module ApplicationHelper
     project_tree(projects) do |project, level|
       name_prefix = (level > 0 ? '&nbsp;' * 2 * level + '&#187; ' : '').html_safe
       tag_options = {:value => project.id}
-      if project == options[:selected] || (options[:selected].respond_to?(:include?) && options[:selected].include?(project))
+      if project == options[:selected] || (options[:selected].respond_to?(:include?) &&
+           options[:selected].include?(project))
         tag_options[:selected] = 'selected'
       else
         tag_options[:selected] = nil
@@ -562,7 +607,17 @@ module ApplicationHelper
   def principals_check_box_tags(name, principals)
     s = +''
     principals.each do |principal|
-      s << "<label>#{ check_box_tag name, principal.id, false, :id => nil } <span class='name icon icon-#{principal.class.name.downcase}'></span>#{h principal}</label>\n"
+      s <<
+        content_tag(
+          'label',
+          check_box_tag(name, principal.id, false, :id => nil) +
+            (avatar(principal, :size => 16).presence ||
+               content_tag(
+                 'span', nil,
+                 :class => "name icon icon-#{principal.class.name.downcase}"
+               )
+            ) + principal
+        )
     end
     s.html_safe
   end
@@ -575,8 +630,11 @@ module ApplicationHelper
     end
     groups = +''
     collection.sort.each do |element|
-      selected_attribute = ' selected="selected"' if option_value_selected?(element, selected) || element.id.to_s == selected
-      (element.is_a?(Group) ? groups : s) << %(<option value="#{element.id}"#{selected_attribute}>#{h element.name}</option>)
+      if option_value_selected?(element, selected) || element.id.to_s == selected
+        selected_attribute = ' selected="selected"'
+      end
+      (element.is_a?(Group) ? groups : s) <<
+        %(<option value="#{element.id}"#{selected_attribute}>#{h element.name}</option>)
     end
     unless groups.empty?
       s << %(<optgroup label="#{h(l(:label_group_plural))}">#{groups}</optgroup>)
@@ -607,7 +665,10 @@ module ApplicationHelper
   end
 
   def html_hours(text)
-    text.gsub(%r{(\d+)([\.:])(\d+)}, '<span class="hours hours-int">\1</span><span class="hours hours-dec">\2\3</span>').html_safe
+    text.gsub(
+      %r{(\d+)([\.:])(\d+)},
+      '<span class="hours hours-int">\1</span><span class="hours hours-dec">\2\3</span>'
+    ).html_safe
   end
 
   def authoring(created, author, options={})
@@ -617,7 +678,9 @@ module ApplicationHelper
   def time_tag(time)
     text = distance_of_time_in_words(Time.now, time)
     if @project
-      link_to(text, project_activity_path(@project, :from => User.current.time_to_date(time)), :title => format_time(time))
+      link_to(text,
+              project_activity_path(@project, :from => User.current.time_to_date(time)),
+              :title => format_time(time))
     else
       content_tag('abbr', text, :title => format_time(time))
     end
@@ -671,7 +734,10 @@ module ApplicationHelper
           b << "\xe2\x80\xa6"
           ancestors = ancestors[-2, 2]
         end
-        b += ancestors.collect {|p| link_to_project(p, {:jump => current_menu_item}, :class => 'ancestor') }
+        b +=
+          ancestors.collect do |p|
+            link_to_project(p, {:jump => current_menu_item}, :class => 'ancestor')
+          end
       end
       b << content_tag(:span, h(@project), class: 'current-project')
       if b.size > 1
@@ -717,7 +783,9 @@ module ApplicationHelper
   def actions_dropdown(&block)
     content = capture(&block)
     if content.present?
-      trigger = content_tag('span', l(:button_actions), :class => 'icon-only icon-actions', :title => l(:button_actions))
+      trigger =
+        content_tag('span', l(:button_actions), :class => 'icon-only icon-actions',
+                    :title => l(:button_actions))
       trigger = content_tag('span', trigger, :class => 'drdn-trigger')
       content = content_tag('div', content, :class => 'drdn-items')
       content = content_tag('div', content, :class => 'drdn-content')
@@ -748,6 +816,7 @@ module ApplicationHelper
     @used_accesskeys ||= []
     key = Redmine::AccessKeys.key_for(s)
     return nil if @used_accesskeys.include?(key)
+
     @used_accesskeys << key
     key
   end
@@ -770,6 +839,7 @@ module ApplicationHelper
       raise ArgumentError, 'invalid arguments to textilizable'
     end
     return '' if text.blank?
+
     project = options[:project] || @project || (obj && obj.respond_to?(:project) ? obj.project : nil)
     @only_path = only_path = options.delete(:only_path) == false ? false : true
 
@@ -855,7 +925,7 @@ module ApplicationHelper
         # search for the picture in attachments
         if found = Attachment.latest_attach(attachments, CGI.unescape(filename))
           image_url = download_named_attachment_url(found, found.filename, :only_path => only_path)
-          desc = found.description.to_s.gsub('"', '')
+          desc = found.description.to_s.delete('"')
           if !desc.blank? && alttext.blank?
             alt = " title=\"#{desc}\" alt=\"#{desc}\""
           end
@@ -918,14 +988,21 @@ module ApplicationHelper
                 "##{page.present? ? Wiki.titleize(page) : title}" + (anchor.present? ? "_#{anchor}" : '')
               else
                 wiki_page_id = page.present? ? Wiki.titleize(page) : nil
-                parent = wiki_page.nil? && obj.is_a?(WikiContent) && obj.page && project == link_project ? obj.page.title : nil
+                parent =
+                  if wiki_page.nil? && obj.is_a?(WikiContent) &&
+                       obj.page && project == link_project
+                    obj.page.title
+                  else
+                    nil
+                  end
                 url_for(:only_path => only_path, :controller => 'wiki',
                         :action => 'show', :project_id => link_project,
                         :id => wiki_page_id, :version => nil, :anchor => anchor,
                         :parent => parent)
               end
             end
-          link_to(title.present? ? title.html_safe : h(page), url, :class => ('wiki-page' + (wiki_page ? '' : ' new')))
+          link_to(title.present? ? title.html_safe : h(page),
+                  url, :class => ('wiki-page' + (wiki_page ? '' : ' new')))
         else
           # project or wiki doesn't exist
           all
@@ -1054,7 +1131,9 @@ module ApplicationHelper
               end
             when 'document'
               if document = Document.visible.find_by_id(oid)
-                link = link_to(document.title, document_url(document, :only_path => only_path), :class => 'document')
+                link = link_to(document.title,
+                               document_url(document, :only_path => only_path),
+                               :class => 'document')
               end
             when 'version'
               if version = Version.visible.find_by_id(oid)
@@ -1066,7 +1145,9 @@ module ApplicationHelper
               end
             when 'forum'
               if board = Board.visible.find_by_id(oid)
-                link = link_to(board.name, project_board_url(board.project, board, :only_path => only_path), :class => 'board')
+                link = link_to(board.name,
+                               project_board_url(board.project, board, :only_path => only_path),
+                               :class => 'board')
               end
             when 'news'
               if news = News.visible.find_by_id(oid)
@@ -1085,7 +1166,9 @@ module ApplicationHelper
             case prefix
             when 'document'
               if project && document = project.documents.visible.find_by_title(name)
-                link = link_to(document.title, document_url(document, :only_path => only_path), :class => 'document')
+                link = link_to(document.title,
+                               document_url(document, :only_path => only_path),
+                               :class => 'document')
               end
             when 'version'
               if project && version = project.versions.visible.find_by_name(name)
@@ -1093,7 +1176,9 @@ module ApplicationHelper
               end
             when 'forum'
               if project && board = project.boards.visible.find_by_name(name)
-                link = link_to(board.name, project_board_url(board.project, board, :only_path => only_path), :class => 'board')
+                link = link_to(board.name,
+                               project_board_url(board.project, board, :only_path => only_path),
+                               :class => 'board')
               end
             when 'news'
               if project && news = project.news.visible.find_by_title(name)
@@ -1109,15 +1194,23 @@ module ApplicationHelper
                   repository = project.repository
                 end
                 if prefix == 'commit'
-                  if repository && (changeset = Changeset.visible.where("repository_id = ? AND scmid LIKE ?", repository.id, "#{name}%").first)
-                    link = link_to(
-                             h("#{project_prefix}#{repo_prefix}#{name}"),
-                             {:only_path => only_path, :controller => 'repositories',
-                              :action => 'revision', :id => project,
-                              :repository_id => repository.identifier_param,
-                              :rev => changeset.identifier},
-                             :class => 'changeset',
-                             :title => truncate_single_line_raw(changeset.comments, 100))
+                  if repository &&
+                       (changeset =
+                          Changeset.visible.
+                            where(
+                              "repository_id = ? AND scmid LIKE ?",
+                              repository.id, "#{name}%"
+                            ).first)
+                    link =
+                      link_to(
+                        h("#{project_prefix}#{repo_prefix}#{name}"),
+                        {:only_path => only_path, :controller => 'repositories',
+                         :action => 'revision', :id => project,
+                         :repository_id => repository.identifier_param,
+                        :rev => changeset.identifier},
+                        :class => 'changeset',
+                        :title => truncate_single_line_raw(changeset.comments, 100)
+                      )
                   end
                 else
                   if repository && User.current.allowed_to?(:browse_repository, project)
@@ -1190,7 +1283,7 @@ module ApplicationHelper
               )|
               (
               (?<sep4>@)
-              (?<identifier3>[A-Za-z0-9_\-@\.]*)
+              (?<identifier3>[A-Za-z0-9_\-@\.]*?)
               )
             )
             (?=
@@ -1205,6 +1298,7 @@ module ApplicationHelper
 
   def parse_sections(text, project, obj, attr, only_path, options)
     return unless options[:edit_section_links]
+
     text.gsub!(HEADING_RE) do
       heading, level = $1, $2
       @current_section += 1
@@ -1235,33 +1329,40 @@ module ApplicationHelper
       item = strip_tags(content).strip
       anchor = sanitize_anchor_name(item)
       # used for single-file wiki export
-      anchor = "#{obj.page.title}_#{anchor}" if options[:wiki_links] == :anchor && (obj.is_a?(WikiContent) || obj.is_a?(WikiContent::Version))
+      if options[:wiki_links] == :anchor && (obj.is_a?(WikiContent) ||
+           obj.is_a?(WikiContent::Version))
+        anchor = "#{obj.page.title}_#{anchor}"
+      end
       @heading_anchors[anchor] ||= 0
       idx = (@heading_anchors[anchor] += 1)
       if idx > 1
         anchor = "#{anchor}-#{idx}"
       end
       @parsed_headings << [level, anchor, item]
-      "<a name=\"#{anchor}\"></a>\n<h#{level} #{attrs}>#{content}<a href=\"##{anchor}\" class=\"wiki-anchor\">&para;</a></h#{level}>"
+      "<a name=\"#{anchor}\"></a>\n<h#{level} #{attrs}>#{content}" \
+        "<a href=\"##{anchor}\" class=\"wiki-anchor\">&para;</a></h#{level}>"
     end
   end
 
-  MACROS_RE = /(
-                (!)?                        # escaping
-                (
-                \{\{                        # opening tag
-                ([\w]+)                     # macro name
-                (\(([^\n\r]*?)\))?          # optional arguments
-                ([\n\r].*?[\n\r])?          # optional block of text
-                \}\}                        # closing tag
-                )
-               )/mx unless const_defined?(:MACROS_RE)
-
-  MACRO_SUB_RE = /(
-                  \{\{
-                  macro\((\d+)\)
-                  \}\}
-                  )/x unless const_defined?(:MACRO_SUB_RE)
+  unless const_defined?(:MACROS_RE)
+    MACROS_RE = /(
+                  (!)?                        # escaping
+                  (
+                  \{\{                        # opening tag
+                  ([\w]+)                     # macro name
+                  (\(([^\n\r]*?)\))?          # optional arguments
+                  ([\n\r].*?[\n\r])?          # optional block of text
+                  \}\}                        # closing tag
+                  )
+                 )/mx
+  end
+  unless const_defined?(:MACRO_SUB_RE)
+    MACRO_SUB_RE = /(
+                    \{\{
+                    macro\((\d+)\)
+                    \}\}
+                    )/x
+  end
 
   # Extracts macros from text
   def catch_macros(text)
@@ -1352,16 +1453,16 @@ module ApplicationHelper
     args << {} unless args.last.is_a?(Hash)
     options = args.last
     if args.first.is_a?(Symbol)
-      options.merge!(:as => args.shift)
+      options[:as] = args.shift
     end
-    options.merge!({:builder => Redmine::Views::LabelledFormBuilder})
+    options[:builder] = Redmine::Views::LabelledFormBuilder
     form_for(*args, &proc)
   end
 
   def labelled_fields_for(*args, &proc)
     args << {} unless args.last.is_a?(Hash)
     options = args.last
-    options.merge!({:builder => Redmine::Views::LabelledFormBuilder})
+    options[:builder] = Redmine::Views::LabelledFormBuilder
     fields_for(*args, &proc)
   end
 
@@ -1496,7 +1597,9 @@ module ApplicationHelper
 
   def calendar_for(field_id)
     include_calendar_headers_tags
-    javascript_tag("$(function() { $('##{field_id}').addClass('date').datepickerFallback(datepickerOptions); });")
+    javascript_tag(
+      "$(function() { $('##{field_id}').addClass('date').datepickerFallback(datepickerOptions); });"
+    )
   end
 
   def include_calendar_headers_tags
@@ -1509,13 +1612,15 @@ module ApplicationHelper
         # Redmine uses 1..7 (monday..sunday) in settings and locales
         # JQuery uses 0..6 (sunday..saturday), 7 needs to be changed to 0
         start_of_week = start_of_week.to_i % 7
-        tags << javascript_tag(
-                   "var datepickerOptions={dateFormat: 'yy-mm-dd', firstDay: #{start_of_week}, " +
-                     "showOn: 'button', buttonImageOnly: true, buttonImage: '" +
-                     path_to_image('/images/calendar.png') +
-                     "', showButtonPanel: true, showWeek: true, showOtherMonths: true, " +
-                     "selectOtherMonths: true, changeMonth: true, changeYear: true, " +
-                     "beforeShow: beforeShowDatePicker};")
+        tags <<
+          javascript_tag(
+            "var datepickerOptions={dateFormat: 'yy-mm-dd', firstDay: #{start_of_week}, " \
+              "showOn: 'button', buttonImageOnly: true, buttonImage: '" +
+            path_to_image('/images/calendar.png') +
+              "', showButtonPanel: true, showWeek: true, showOtherMonths: true, " \
+              "selectOtherMonths: true, changeMonth: true, changeYear: true, " \
+              "beforeShow: beforeShowDatePicker};"
+          )
         jquery_locale = l('jquery.locale', :default => current_language.to_s)
         unless jquery_locale == 'en'
           tags << javascript_include_tag("i18n/datepicker-#{jquery_locale}.js")
@@ -1596,9 +1701,21 @@ module ApplicationHelper
 
   # Returns the javascript tags that are included in the html layout head
   def javascript_heads
-    tags = javascript_include_tag('jquery-2.2.4-ui-1.11.0-ujs-5.2.3', 'tribute-3.7.3.min', 'application', 'responsive')
+    tags = javascript_include_tag(
+      'jquery-3.5.1-ui-1.12.1-ujs-5.2.4.5',
+      'jquery-migrate-3.3.2.min.js',
+      'tribute-5.1.3.min',
+      'tablesort-5.2.1.min.js',
+      'tablesort-5.2.1.number.min.js',
+      'application',
+      'responsive')
     unless User.current.pref.warn_on_leaving_unsaved == '0'
-      tags << "\n".html_safe + javascript_tag("$(window).on('load', function(){ warnLeavingUnsaved('#{escape_javascript l(:text_warn_on_leaving_unsaved)}'); });")
+      warn_text = escape_javascript(l(:text_warn_on_leaving_unsaved))
+      tags <<
+        "\n".html_safe +
+          javascript_tag(
+            "$(window).on('load', function(){ warnLeavingUnsaved('#{warn_text}'); });"
+          )
     end
     tags
   end
@@ -1618,7 +1735,7 @@ module ApplicationHelper
     # TODO: use #image_url introduced in Rails4
     path = favicon_path
     base = url_for(:controller => 'welcome', :action => 'index', :only_path => false)
-    base.sub(%r{/+$},'') + '/' + path.sub(%r{^/+},'')
+    base.sub(%r{/+$}, '') + '/' + path.sub(%r{^/+}, '')
   end
 
   def robot_exclusion_tag
@@ -1649,6 +1766,7 @@ module ApplicationHelper
 
   def export_csv_encoding_select_tag
     return if l(:general_csv_encoding).casecmp('UTF-8') == 0
+
     options = [l(:general_csv_encoding), 'UTF-8']
     content_tag(:p) do
       concat(
@@ -1669,11 +1787,45 @@ module ApplicationHelper
         messages[message] << item
       end
     end
-    messages.map { |message, items|
+    messages.map do |message, items|
       "#{message}: " + items.map {|i| "##{i.id}"}.join(', ')
+    end
+  end
+
+  def render_if_exist(options = {}, locals = {}, &block)
+    if options[:partial]
+      if lookup_context.exists?(options[:partial], lookup_context.prefixes, true)
+        render(options, locals, &block)
+      end
+    else
+      render(options, locals, &block)
+    end
+  end
+
+  def autocomplete_data_sources(project)
+    {
+      issues: auto_complete_issues_path(:project_id => project, :q => ''),
+      wiki_pages: auto_complete_wiki_pages_path(:project_id => project, :q => '')
     }
   end
 
+  def heads_for_auto_complete(project)
+    data_sources = autocomplete_data_sources(project)
+    javascript_tag(
+      "rm = window.rm || {};" \
+      "rm.AutoComplete = rm.AutoComplete || {};" \
+      "rm.AutoComplete.dataSources = '#{data_sources.to_json}';"
+    )
+  end
+
+  def copy_object_url_link(url)
+    link_to_function(
+      l(:button_copy_link), 'copyTextToClipboard(this);',
+      class: 'icon icon-copy-link',
+      data: {'clipboard-text' => url}
+    )
+  end
+
   private
 
   def wiki_helper
