@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -60,16 +60,39 @@ module UsersHelper
     end
   end
 
+  def user_emails(user)
+    emails = [user.mail]
+    emails += user.email_addresses.order(:id).where(:is_default => false).pluck(:address)
+    emails.map {|email| mail_to(email, nil)}.join(', ').html_safe
+  end
+
   def user_settings_tabs
-    tabs = [{:name => 'general', :partial => 'users/general', :label => :label_general},
-            {:name => 'memberships', :partial => 'users/memberships', :label => :label_project_plural}
-            ]
+    tabs =
+      [
+        {:name => 'general', :partial => 'users/general', :label => :label_general},
+        {:name => 'memberships', :partial => 'users/memberships', :label => :label_project_plural}
+      ]
     if Group.givable.any?
       tabs.insert 1, {:name => 'groups', :partial => 'users/groups', :label => :label_group_plural}
     end
     tabs
   end
 
+  def csv_content(column_name, user)
+    case column_name
+    when 'status'
+      l("status_#{User::LABEL_BY_STATUS[user.status]}")
+    when 'twofa_scheme'
+      if user.twofa_active?
+        l("twofa__#{user.twofa_scheme}__name")
+      else
+        l(:label_disabled)
+      end
+    else
+      user.send(column_name)
+    end
+  end
+
   def users_to_csv(users)
     Redmine::Export::CSV.generate(:encoding => params[:encoding]) do |csv|
       columns = [
@@ -78,20 +101,31 @@ module UsersHelper
         'lastname',
         'mail',
         'admin',
+        'status',
+        'twofa_scheme',
         'created_on',
+        'updated_on',
         'last_login_on',
-        'status'
+        'passwd_changed_on'
       ]
+      user_custom_fields = UserCustomField.sorted
 
       # csv header fields
-      csv << columns.map{|column| l('field_' + column)}
+      csv << columns.map {|column| l('field_' + column)} + user_custom_fields.pluck(:name)
       # csv lines
+      users = users.preload(:custom_values)
       users.each do |user|
-        csv << columns.map do |column|
-          if column == 'status'
-            l(("status_#{User::LABEL_BY_STATUS[user.status]}"))
-          else
-            format_object(user.send(column), false)
+        values = columns.map {|c| csv_content(c, user)} +
+                 user_custom_fields.map {|custom_field| user.custom_value_for(custom_field)}
+
+        csv << values.map do |value|
+          format_object(value, false) do |v|
+            case v.class.name
+            when 'Float'
+              sprintf('%.2f', v).gsub('.', l(:general_csv_decimal_separator))
+            else
+              v
+            end
           end
         end
       end
