@@ -89,7 +89,7 @@ module Redmine
           i = 0
           l = 1
           inside_pre = false
-          @text.split(/(^(?:.+\r?\n\r?(?:\=+|\-+)|#+.+|(?:~~~|```).*)\s*$)/).each do |part|
+          @text.split(/(^(?:\S+\r?\n\r?(?:\=+|\-+)|#+.+|(?:~~~|```).*)\s*$)/).each do |part|
             level = nil
             if part =~ /\A(~{3,}|`{3,})(\s*\S+)?\s*$/
               if !inside_pre
