@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -50,9 +50,7 @@ class ProjectsControllerTest < Redmine::ControllerTest
   end
 
   def test_index_atom
-    get :index, :params => {
-        :format => 'atom'
-      }
+    get(:index, :params => {:format => 'atom'})
     assert_response :success
     assert_select 'feed>title', :text => 'Redmine: Latest projects'
     assert_select 'feed>entry', :count => Project.visible(User.current).count
@@ -98,7 +96,7 @@ class ProjectsControllerTest < Redmine::ControllerTest
 
   def test_index_as_list_should_format_column_value
     get :index, :params => {
-      :c => ['name', 'status', 'short_description', 'homepage', 'parent_id', 'identifier', 'is_public', 'created_on', 'project.cf_3'],
+      :c => ['name', 'status', 'short_description', 'homepage', 'parent_id', 'identifier', 'is_public', 'created_on', 'cf_3'],
       :display_type => 'list'
     }
     assert_response :success
@@ -113,7 +111,7 @@ class ProjectsControllerTest < Redmine::ControllerTest
         assert_select 'td.identifier', :text => 'ecookbook'
         assert_select 'td.is_public', :text => 'Yes'
         assert_select 'td.created_on', :text => format_time(project.created_on)
-        assert_select 'td.project_cf_3.list', :text => 'Stable'
+        assert_select 'td.cf_3.list', :text => 'Stable'
       end
       assert_select 'tr[id=?]', 'project-4' do
         assert_select 'td.parent_id a[href=?]', '/projects/ecookbook', :text => 'eCookbook'
@@ -209,24 +207,71 @@ class ProjectsControllerTest < Redmine::ControllerTest
     end
   end
 
+  def test_index_sort_by_custom_field
+    @request.session[:user_id] = 1
+
+    cf = ProjectCustomField.find(3)
+    CustomValue.create!(:custom_field => cf, :customized => Project.find(2), :value => 'Beta')
+
+    get(
+      :index,
+      :params => {
+        :display_type => 'list',
+        :c => ['name', 'identifier', 'cf_3'],
+        :set_filter => 1,
+        :sort => "cf_#{cf.id}:asc"
+      }
+    )
+    assert_response :success
+
+    assert_equal(
+      ['Beta', 'Stable'],
+      columns_values_in_list('cf_3').reject {|p| p.empty?}
+    )
+  end
+
+  def test_index_with_int_custom_field_total
+    @request.session[:user_id] = 1
+
+    field = ProjectCustomField.generate!(:field_format => 'int')
+    CustomValue.create!(:customized => Project.find(1), :custom_field => field, :value => '2')
+    CustomValue.create!(:customized => Project.find(2), :custom_field => field, :value => '7')
+    get(
+      :index,
+      :params => {
+        :display_type => 'list',
+        :t => ["cf_#{field.id}"]
+      }
+    )
+    assert_response :success
+    assert_select '.query-totals'
+    assert_select ".total-for-cf-#{field.id} span.value", :text => '9'
+  end
+
   def test_autocomplete_js
-    get :autocomplete, :params => {
+    get(
+      :autocomplete,
+      :params => {
         :format => 'js',
         :q => 'coo'
       },
       :xhr => true
+    )
     assert_response :success
-    assert_equal 'text/javascript', response.content_type
+    assert_equal 'text/javascript', response.media_type
   end
 
   def test_autocomplete_js_with_blank_search_term
-    get :autocomplete, :params => {
+    get(
+      :autocomplete,
+      :params => {
         :format => 'js',
         :q => ''
       },
       :xhr => true
+    )
     assert_response :success
-    assert_equal 'text/javascript', response.content_type
+    assert_equal 'text/javascript', response.media_type
   end
 
   test "#index by non-admin user with view_time_entries permission should show overall spent time link" do
@@ -276,10 +321,7 @@ class ProjectsControllerTest < Redmine::ControllerTest
     Role.find(1).remove_permission! :add_project
     Role.find(1).add_permission! :add_subprojects
     @request.session[:user_id] = 2
-
-    get :new, :params => {
-        :parent_id => 'ecookbook'
-      }
+    get(:new, :params => {:parent_id => 'ecookbook'})
     assert_response :success
 
     assert_select 'select[name=?]', 'project[parent_id]' do
@@ -317,8 +359,9 @@ class ProjectsControllerTest < Redmine::ControllerTest
 
   test "#create by admin user should create a new project" do
     @request.session[:user_id] = 1
-
-    post :create, :params => {
+    post(
+      :create,
+      :params => {
         :project => {
           :name => "blog",
           :description => "weblog",
@@ -334,6 +377,7 @@ class ProjectsControllerTest < Redmine::ControllerTest
           :enabled_module_names => ['issue_tracking', 'news', 'repository']
         }
       }
+    )
     assert_redirected_to '/projects/blog/settings'
 
     project = Project.find_by_name('blog')
@@ -351,9 +395,10 @@ class ProjectsControllerTest < Redmine::ControllerTest
 
   test "#create by admin user should create a new subproject" do
     @request.session[:user_id] = 1
-
     assert_difference 'Project.count' do
-      post :create, :params => {
+      post(
+        :create,
+        :params => {
           :project => {
             :name => "blog",
             :description => "weblog",
@@ -363,9 +408,9 @@ class ProjectsControllerTest < Redmine::ControllerTest
               '3' => 'Beta'
             },
             :parent_id => 1
-
           }
         }
+      )
       assert_redirected_to '/projects/blog/settings'
     end
 
@@ -376,15 +421,17 @@ class ProjectsControllerTest < Redmine::ControllerTest
 
   test "#create by admin user should continue" do
     @request.session[:user_id] = 1
-
     assert_difference 'Project.count' do
-      post :create, :params => {
+      post(
+        :create,
+        :params => {
           :project => {
             :name => "blog",
             :identifier => "blog"
           },
           :continue => 'Create and continue'
         }
+      )
     end
     assert_redirected_to '/projects/new'
   end
@@ -392,8 +439,8 @@ class ProjectsControllerTest < Redmine::ControllerTest
   test "#create by non-admin user with add_project permission should create a new project" do
     Role.non_member.add_permission! :add_project
     @request.session[:user_id] = 9
-
-    post :create, :params => {
+    post(
+      :create, :params => {
         :project => {
           :name => "blog",
           :description => "weblog",
@@ -404,10 +451,9 @@ class ProjectsControllerTest < Redmine::ControllerTest
           },
           :tracker_ids => ['1', '3'],
           :enabled_module_names => ['issue_tracking', 'news', 'repository']
-
         }
       }
-
+    )
     assert_redirected_to '/projects/blog/settings'
 
     project = Project.find_by_name('blog')
@@ -426,9 +472,10 @@ class ProjectsControllerTest < Redmine::ControllerTest
     Role.non_member.add_permission! :add_project
     User.find(9).update! :language => 'en'
     @request.session[:user_id] = 9
-
     assert_no_difference 'Project.count' do
-      post :create, :params => {
+      post(
+        :create,
+        :params => {
           :project => {
             :name => "blog",
             :description => "weblog",
@@ -438,9 +485,9 @@ class ProjectsControllerTest < Redmine::ControllerTest
               '3' => 'Beta'
             },
             :parent_id => 1
-
           }
         }
+      )
     end
     assert_response :success
     assert_select_error /Subproject of is invalid/
@@ -450,8 +497,9 @@ class ProjectsControllerTest < Redmine::ControllerTest
     Role.find(1).remove_permission! :add_project
     Role.find(1).add_permission! :add_subprojects
     @request.session[:user_id] = 2
-
-    post :create, :params => {
+    post(
+      :create,
+      :params => {
         :project => {
           :name => "blog",
           :description => "weblog",
@@ -461,9 +509,9 @@ class ProjectsControllerTest < Redmine::ControllerTest
             '3' => 'Beta'
           },
           :parent_id => 1
-
         }
       }
+    )
     assert_redirected_to '/projects/blog/settings'
     project = Project.find_by_name('blog')
     assert_equal 1, project.parent_id
@@ -473,9 +521,10 @@ class ProjectsControllerTest < Redmine::ControllerTest
     Role.find(1).remove_permission! :add_project
     Role.find(1).add_permission! :add_subprojects
     @request.session[:user_id] = 2
-
     assert_no_difference 'Project.count' do
-      post :create, :params => {
+      post(
+        :create,
+        :params => {
           :project => {
             :name => "blog",
             :description => "weblog",
@@ -484,9 +533,9 @@ class ProjectsControllerTest < Redmine::ControllerTest
             :custom_field_values => {
               '3' => 'Beta'
             }
-
           }
         }
+      )
     end
     assert_response :success
     assert_select_error /Subproject of is invalid/
@@ -496,10 +545,11 @@ class ProjectsControllerTest < Redmine::ControllerTest
     Role.find(1).remove_permission! :add_project
     Role.find(1).add_permission! :add_subprojects
     @request.session[:user_id] = 2
-
     assert !User.find(2).member_of?(Project.find(6))
     assert_no_difference 'Project.count' do
-      post :create, :params => {
+      post(
+        :create,
+        :params => {
           :project => {
             :name => "blog",
             :description => "weblog",
@@ -509,9 +559,9 @@ class ProjectsControllerTest < Redmine::ControllerTest
               '3' => 'Beta'
             },
             :parent_id => 6
-
           }
         }
+      )
     end
     assert_response :success
     assert_select_error /Subproject of is invalid/
@@ -522,30 +572,33 @@ class ProjectsControllerTest < Redmine::ControllerTest
     default_role = Role.generate!(:permissions => [:view_issues, :add_project])
     user = User.generate!
     @request.session[:user_id] = user.id
-
     with_settings :new_project_user_role_id => default_role.id.to_s, :default_projects_modules => %w(news files) do
       project = new_record(Project) do
-        post :create, :params => {
+        post(
+          :create,
+          :params => {
             :project => {
               :name => "blog1",
               :identifier => "blog1",
               :enabled_module_names => ["issue_tracking", "repository"]
-
             }
           }
+        )
       end
       assert_equal %w(files news), project.enabled_module_names.sort
 
       default_role.add_permission!(:select_project_modules)
       project = new_record(Project) do
-        post :create, :params => {
+        post(
+          :create,
+          :params => {
             :project => {
               :name => "blog2",
               :identifier => "blog2",
               :enabled_module_names => ["issue_tracking", "repository"]
-
             }
           }
+        )
       end
       assert_equal %w(issue_tracking repository), project.enabled_module_names.sort
     end
@@ -555,20 +608,20 @@ class ProjectsControllerTest < Redmine::ControllerTest
     Role.find_by_name('Manager').add_permission! :add_subprojects
     parent = Project.find(1)
     @request.session[:user_id] = 2
-
     assert_difference 'Project.count' do
-      post :create, :params => {
+      post(
+        :create,
+        :params => {
           :project => {
             :name => 'inherited',
             :identifier => 'inherited',
             :parent_id => parent.id,
             :inherit_members => '1'
-
           }
         }
+      )
       assert_response 302
     end
-
     project = Project.order('id desc').first
     assert_equal 'inherited', project.name
     assert_equal parent, project.parent
@@ -580,14 +633,16 @@ class ProjectsControllerTest < Redmine::ControllerTest
     with_settings :default_projects_modules => ['issue_tracking', 'repository'] do
       @request.session[:user_id] = 1
       assert_no_difference 'Project.count' do
-        post :create, :params => {
+        post(
+          :create,
+          :params => {
             :project => {
               :name => "blog",
               :identifier => "",
               :enabled_module_names => %w(issue_tracking news)
-
             }
           }
+        )
       end
       assert_response :success
       %w(issue_tracking news).each do |mod|
@@ -598,17 +653,13 @@ class ProjectsControllerTest < Redmine::ControllerTest
   end
 
   def test_show_by_id
-    get :show, :params => {
-        :id => 1
-      }
+    get(:show, :params => {:id => 1})
     assert_response :success
     assert_select '#header h1', :text => "eCookbook"
   end
 
   def test_show_by_identifier
-    get :show, :params => {
-        :id => 'ecookbook'
-      }
+    get(:show, :params => {:id => 'ecookbook'})
     assert_response :success
     assert_select '#header h1', :text => "eCookbook"
   end
@@ -617,31 +668,22 @@ class ProjectsControllerTest < Redmine::ControllerTest
     p = Project.find(1)
     p.enabled_module_names = []
     p.save!
-
-    get :show, :params => {
-        :id => 'ecookbook'
-      }
+    get(:show, :params => {:id => 'ecookbook'})
     assert_response :success
     assert_select '#main.nosidebar'
   end
 
   def test_show_should_display_visible_custom_fields
     ProjectCustomField.find_by_name('Development status').update_attribute :visible, true
-    get :show, :params => {
-        :id => 'ecookbook'
-      }
+    get(:show, :params => {:id => 'ecookbook'})
     assert_response :success
-
-    assert_select 'li[class=?]', 'cf_3', :text => /Development status/
+    assert_select 'li.list_cf.cf_3', :text => /Development status/
   end
 
   def test_show_should_not_display_hidden_custom_fields
     ProjectCustomField.find_by_name('Development status').update_attribute :visible, false
-    get :show, :params => {
-        :id => 'ecookbook'
-      }
+    get(:show, :params => {:id => 'ecookbook'})
     assert_response :success
-
     assert_select 'li', :text => /Development status/, :count => 0
   end
 
@@ -649,43 +691,30 @@ class ProjectsControllerTest < Redmine::ControllerTest
     f1 = ProjectCustomField.generate! :field_format => 'list', :possible_values => %w(Foo Bar), :multiple => true
     f2 = ProjectCustomField.generate! :field_format => 'list', :possible_values => %w(Baz Qux), :multiple => true
     project = Project.generate!(:custom_field_values => {f2.id.to_s => %w(Qux)})
-
-    get :show, :params => {
-        :id => project.id
-      }
+    get(:show, :params => {:id => project.id})
     assert_response :success
-
     assert_select 'li', :text => /#{f1.name}/, :count => 0
     assert_select 'li', :text => /#{f2.name}/
   end
 
   def test_show_should_not_display_blank_text_custom_fields
     f1 = ProjectCustomField.generate! :field_format => 'text'
-
-    get :show, :params => {
-        :id => 1
-      }
+    get(:show, :params => {:id => 1})
     assert_response :success
-
     assert_select 'li', :text => /#{f1.name}/, :count => 0
   end
 
   def test_show_should_not_fail_when_custom_values_are_nil
     project = Project.find_by_identifier('ecookbook')
     project.custom_values.first.update_attribute(:value, nil)
-    get :show, :params => {
-        :id => 'ecookbook'
-      }
+    get(:show, :params => {:id => 'ecookbook'})
     assert_response :success
   end
 
   def test_show_archived_project_should_be_denied
     project = Project.find_by_identifier('ecookbook')
     project.archive
-
-    get :show, :params => {
-        :id => 'ecookbook'
-      }
+    get(:show, :params => {:id => 'ecookbook'})
     assert_response 403
     assert_select 'p', :text => /archived/
     assert_not_include project.name, response.body
@@ -695,47 +724,54 @@ class ProjectsControllerTest < Redmine::ControllerTest
     @request.session[:user_id] = 1
     project = Project.find_by_identifier('ecookbook')
     project.archive
-
-    get :show, :params => {
-        :id => 'ecookbook'
-      }
+    get(:show, :params => {:id => 'ecookbook'})
     assert_response 403
     assert_select 'a', :text => "Unarchive"
   end
 
   def test_show_should_not_show_private_subprojects_that_are_not_visible
-    get :show, :params => {
-        :id => 'ecookbook'
-      }
+    get(:show, :params => {:id => 'ecookbook'})
     assert_response :success
     assert_select 'a', :text => /Private child/, :count => 0
   end
 
   def test_show_should_show_private_subprojects_that_are_visible
     @request.session[:user_id] = 2 # manager who is a member of the private subproject
-    get :show, :params => {
-        :id => 'ecookbook'
-      }
+    get(:show, :params => {:id => 'ecookbook'})
     assert_response :success
     assert_select 'a', :text => /Private child/
   end
 
   def test_show_by_member_on_leaf_project_should_display_issue_counts
     @request.session[:user_id] = 2
-    get :show, :params => {
-        :id => 'onlinestore'
-      }
+    get(:show, :params => {:id => 'onlinestore'})
     assert_response :success
     # Make sure there's a > 0 issue count
     assert_select 'table.issue-report td.total a', :text => %r{\A[1-9]\d*\z}
   end
 
+  def test_show_should_not_display_subprojects_trackers_when_subprojects_issues_is_not_displayed
+    project = Project.find('ecookbook')
+    tracker = project.trackers.find_by(name: 'Support request')
+    project.trackers.delete(tracker)
+    @request.session[:user_id] = 2
+
+    with_settings :display_subprojects_issues => '1' do
+      get(:show, :params => {:id => 'ecookbook'})
+      assert_response :success
+      assert_select 'table.issue-report td.name', :text => 'Support request', :count => 1
+    end
+
+    with_settings :display_subprojects_issues => '0' do
+      get(:show, :params => {:id => 'ecookbook'})
+      assert_response :success
+      assert_select 'table.issue-report td.name', :text => 'Support request', :count => 0
+    end
+  end
+
   def test_show_should_spent_and_estimated_time
     @request.session[:user_id] = 1
-    get :show, :params => {
-        :id => 'ecookbook'
-      }
-
+    get(:show, :params => {:id => 'ecookbook'})
     assert_select 'div.spent_time.box>ul' do
       assert_select '>li:nth-child(1)', :text => 'Estimated time: 203.50 hours'
       assert_select '>li:nth-child(2)', :text => 'Spent time: 162.90 hours'
@@ -744,53 +780,42 @@ class ProjectsControllerTest < Redmine::ControllerTest
 
   def test_settings
     @request.session[:user_id] = 2 # manager
-    get :settings, :params => {
-        :id => 1
-      }
+    get(:settings, :params => {:id => 1})
     assert_response :success
-
     assert_select 'input[name=?]', 'project[name]'
   end
 
   def test_settings_of_subproject
     @request.session[:user_id] = 2
-    get :settings, :params => {
-        :id => 'private-child'
-      }
+    get(:settings, :params => {:id => 'private-child'})
     assert_response :success
-
     assert_select 'input[type=checkbox][name=?]', 'project[inherit_members]'
   end
 
   def test_settings_should_be_denied_for_member_on_closed_project
     Project.find(1).close
     @request.session[:user_id] = 2 # manager
-
-    get :settings, :params => {
-        :id => 1
-      }
+    get(:settings, :params => {:id => 1})
     assert_response 403
   end
 
   def test_settings_should_be_denied_for_anonymous_on_closed_project
     Project.find(1).close
-
-    get :settings, :params => {
-        :id => 1
-      }
+    get(:settings, :params => {:id => 1})
     assert_response 403
   end
 
   def test_settings_should_accept_version_status_filter
     @request.session[:user_id] = 2
-
-    get :settings, :params => {
+    get(
+      :settings,
+      :params => {
         :id => 'ecookbook',
         :tab => 'versions',
         :version_status => 'locked'
       }
+    )
     assert_response :success
-
     assert_select 'select[name=version_status]' do
       assert_select 'option[value=locked][selected=selected]'
     end
@@ -803,15 +828,16 @@ class ProjectsControllerTest < Redmine::ControllerTest
 
   def test_settings_should_accept_version_name_filter
     @request.session[:user_id] = 2
-
-    get :settings, :params => {
+    get(
+      :settings,
+      :params => {
         :id => 'ecookbook',
         :tab => 'versions',
         :version_status => '',
         :version_name => '.1'
       }
+    )
     assert_response :success
-
     assert_select 'input[name=version_name][value=?]', '.1'
     assert_select 'table.versions tbody' do
       assert_select 'tr', 1
@@ -824,15 +850,15 @@ class ProjectsControllerTest < Redmine::ControllerTest
     project = Project.find(1)
     project.default_version_id = 3
     project.save!
-
     @request.session[:user_id] = 2
-
-    get :settings, :params => {
+    get(
+      :settings,
+      :params => {
         :id => 'ecookbook',
         :tab => 'versions',
       }
+    )
     assert_response :success
-
     assert_select 'table.versions tbody' do
       # asserts that only one version is marked as default
       assert_select 'td.tick span.icon-checked', 1
@@ -847,11 +873,13 @@ class ProjectsControllerTest < Redmine::ControllerTest
     user.lock!
     assert user.reload.locked?
     @request.session[:user_id] = 2
-
-    get :settings, :params => {
+    get(
+      :settings,
+      :params => {
         :id => 'ecookbook',
         :tab => 'members'
       }
+    )
     assert_response :success
     assert_select "tr#member-#{member.id}"
   end
@@ -860,18 +888,13 @@ class ProjectsControllerTest < Redmine::ControllerTest
     @request.session[:user_id] = 3
     project = Project.find(1)
     role = User.find(3).roles_for_project(project).first
-
     role.permissions = []
     role.save
-    get :settings, :params => {
-      :id => project.id
-    }
+    get(:settings, :params => {:id => project.id})
     assert_response 403
 
     role.add_permission! :manage_repository, :manage_boards, :manage_project_activities
-    get :settings, :params => {
-      :id => project.id
-    }
+    get(:settings, :params => {:id => project.id})
     assert_response :success
     assert_select 'a[id^=tab-]', 3
     assert_select 'a#tab-repositories'
@@ -881,25 +904,51 @@ class ProjectsControllerTest < Redmine::ControllerTest
 
   def test_settings_should_not_display_custom_fields_not_visible_for_user
     @request.session[:user_id] = 2
-
     ProjectCustomField.find_by_name('Development status').update_attribute :visible, false
-    get :settings, :params => {
+    get(
+      :settings,
+      :params => {
         :id => 'ecookbook'
       }
+    )
+    assert_response :success
+    assert_select 'select#project_custom_field_values_3', :count => 0
+  end
+
+  def test_settings_issue_tracking
+    @request.session[:user_id] = 1
+    project = Project.find(1)
+    project.default_version_id = 3
+    project.save!
+
+    get(
+      :settings,
+      :params => {
+        :id => 'ecookbook',
+        :tab => 'issues',
+      }
+    )
     assert_response :success
 
-    assert_select 'select#project_custom_field_values_3', :count => 0
+    assert_select 'form[id=?]', 'project_issue_tracking', 1
+    assert_select 'input[name=?]', 'project[tracker_ids][]'
+    assert_select 'input[name=?]', 'project[issue_custom_field_ids][]'
+    assert_select 'select[name=?]', 'project[default_version_id]', 1
+    assert_select 'select[name=?]', 'project[default_assigned_to_id]', 1
   end
 
   def test_update
     @request.session[:user_id] = 2 # manager
-    post :update, :params => {
+    post(
+      :update,
+      :params => {
         :id => 1,
         :project => {
           :name => 'Test changed name',
           :issue_custom_field_ids => ['']
         }
       }
+    )
     assert_redirected_to '/projects/ecookbook/settings'
     project = Project.find(1)
     assert_equal 'Test changed name', project.name
@@ -907,12 +956,15 @@ class ProjectsControllerTest < Redmine::ControllerTest
 
   def test_update_with_failure
     @request.session[:user_id] = 2 # manager
-    post :update, :params => {
+    post(
+      :update,
+      :params => {
         :id => 1,
         :project => {
           :name => ''
         }
       }
+    )
     assert_response :success
     assert_select_error /name cannot be blank/i
   end
@@ -920,26 +972,30 @@ class ProjectsControllerTest < Redmine::ControllerTest
   def test_update_should_be_denied_for_member_on_closed_project
     Project.find(1).close
     @request.session[:user_id] = 2 # manager
-
-    post :update, :params => {
+    post(
+      :update,
+      :params => {
         :id => 1,
         :project => {
           :name => 'Closed'
         }
       }
+    )
     assert_response 403
     assert_equal 'eCookbook', Project.find(1).name
   end
 
   def test_update_should_be_denied_for_anonymous_on_closed_project
     Project.find(1).close
-
-    post :update, :params => {
+    post(
+      :update,
+      :params => {
         :id => 1,
         :project => {
           :name => 'Closed'
         }
       }
+    )
     assert_response 403
     assert_equal 'eCookbook', Project.find(1).name
   end
@@ -949,13 +1005,15 @@ class ProjectsControllerTest < Redmine::ControllerTest
     user = User.generate!
     User.add_to_project(user, child, Role.generate!(:permissions => [:edit_project]))
     @request.session[:user_id] = user.id
-
-    post :update, :params => {
+    post(
+      :update,
+      :params => {
         :id => child.id,
         :project => {
           :name => 'Updated'
         }
       }
+    )
     assert_response 302
     assert_match /Successful update/, flash[:notice]
   end
@@ -964,23 +1022,58 @@ class ProjectsControllerTest < Redmine::ControllerTest
     @request.session[:user_id] = 2
     Project.find(1).enabled_module_names = ['issue_tracking', 'news']
 
-    post :update, :params => {
+    post(
+      :update,
+      :params => {
         :id => 1,
         :project => {
           :enabled_module_names => ['issue_tracking', 'repository', 'documents']
         }
       }
+    )
     assert_redirected_to '/projects/ecookbook/settings'
     assert_equal ['documents', 'issue_tracking', 'repository'], Project.find(1).enabled_module_names.sort
   end
 
+  def test_update_custom_field_should_update_updated_on
+    @request.session[:user_id] = 2
+    project = Project.find(1)
+    project.update_attribute :updated_on, nil
+    assert_equal 'Stable', project.custom_value_for(3).value
+
+    travel_to Time.current do
+      post(
+        :update,
+        :params => {
+          :id => 1,
+          :project => {
+            :custom_field_values => {'3' => 'Alpha'}
+          }
+        }
+      )
+      assert_redirected_to '/projects/ecookbook/settings'
+      assert_equal 'Successful update.', flash[:notice]
+      project.reload
+      assert_equal 'Alpha', project.custom_value_for(3).value
+      assert_equal Time.current, project.updated_on
+    end
+  end
+
   def test_destroy_leaf_project_without_confirmation_should_show_confirmation
     @request.session[:user_id] = 1 # admin
 
     assert_no_difference 'Project.count' do
-      delete :destroy, :params => {
-          :id => 2
-        }
+      delete(:destroy, :params => {:id => 2})
+      assert_response :success
+    end
+    assert_select '.warning', :text => /Are you sure you want to delete this project/
+  end
+
+  def test_destroy_leaf_project_with_wrong_confirmation_should_show_confirmation
+    @request.session[:user_id] = 1 # admin
+
+    assert_no_difference 'Project.count' do
+      delete(:destroy, :params => {:id => 2, :confirm => 'wrong'})
       assert_response :success
     end
     assert_select '.warning', :text => /Are you sure you want to delete this project/
@@ -991,9 +1084,7 @@ class ProjectsControllerTest < Redmine::ControllerTest
     @request.session[:user_id] = 1 # admin
 
     assert_no_difference 'Project.count' do
-      delete :destroy, :params => {
-          :id => 1
-        }
+      delete(:destroy, :params => {:id => 1})
       assert_response :success
     end
     assert_select 'strong',
@@ -1007,20 +1098,55 @@ class ProjectsControllerTest < Redmine::ControllerTest
     @request.session[:user_id] = 1 # admin
 
     assert_difference 'Project.count', -5 do
-      delete :destroy, :params => {
+      delete(
+        :destroy,
+        :params => {
           :id => 1,
-          :confirm => 1
+          :confirm => 'ecookbook'
         }
+      )
       assert_redirected_to '/admin/projects'
     end
     assert_nil Project.find_by_id(1)
   end
 
+  def test_destroy_with_normal_user_should_destroy
+    set_tmp_attachments_directory
+    @request.session[:user_id] = 2 # non-admin
+
+    assert_difference 'Project.count', -1 do
+      delete(
+        :destroy,
+        :params => {
+          :id => 2,
+          :confirm => 'onlinestore'
+        }
+      )
+      assert_redirected_to '/projects'
+    end
+    assert_nil Project.find_by_id(2)
+  end
+
+  def test_destroy_with_normal_user_should_not_destroy_with_subprojects
+    set_tmp_attachments_directory
+    @request.session[:user_id] = 2 # non-admin
+
+    assert_difference 'Project.count', 0 do
+      delete(
+        :destroy,
+        :params => {
+          :id => 1,
+          :confirm => 'ecookbook'
+        }
+      )
+      assert_response 403
+    end
+    assert Project.find(1)
+  end
+
   def test_archive
     @request.session[:user_id] = 1 # admin
-    post :archive, :params => {
-        :id => 1
-      }
+    post(:archive, :params => {:id => 1})
     assert_redirected_to '/admin/projects'
     assert !Project.find(1).active?
   end
@@ -1028,9 +1154,7 @@ class ProjectsControllerTest < Redmine::ControllerTest
   def test_archive_with_failure
     @request.session[:user_id] = 1
     Project.any_instance.stubs(:archive).returns(false)
-    post :archive, :params => {
-        :id => 1
-      }
+    post(:archive, :params => {:id => 1})
     assert_redirected_to '/admin/projects'
     assert_match /project cannot be archived/i, flash[:error]
   end
@@ -1038,18 +1162,14 @@ class ProjectsControllerTest < Redmine::ControllerTest
   def test_unarchive
     @request.session[:user_id] = 1 # admin
     Project.find(1).archive
-    post :unarchive, :params => {
-        :id => 1
-      }
+    post(:unarchive, :params => {:id => 1})
     assert_redirected_to '/admin/projects'
     assert Project.find(1).active?
   end
 
   def test_close
     @request.session[:user_id] = 2
-    post :close, :params => {
-        :id => 1
-      }
+    post(:close, :params => {:id => 1})
     assert_redirected_to '/projects/ecookbook'
     assert_equal Project::STATUS_CLOSED, Project.find(1).status
   end
@@ -1057,9 +1177,7 @@ class ProjectsControllerTest < Redmine::ControllerTest
   def test_reopen
     Project.find(1).close
     @request.session[:user_id] = 2
-    post :reopen, :params => {
-        :id => 1
-      }
+    post(:reopen, :params => {:id => 1})
     assert_redirected_to '/projects/ecookbook'
     assert Project.find(1).active?
   end
@@ -1069,9 +1187,7 @@ class ProjectsControllerTest < Redmine::ControllerTest
     parent = nil
     6.times do |i|
       p = Project.generate_with_parent!(parent)
-      get :show, :params => {
-          :id => p
-        }
+      get(:show, :params => {:id => p})
       assert_select '#header h1' do
         assert_select 'a', :count => [i, 3].min
       end
@@ -1083,10 +1199,7 @@ class ProjectsControllerTest < Redmine::ControllerTest
   def test_get_copy
     @request.session[:user_id] = 1 # admin
     orig = Project.find(1)
-
-    get :copy, :params => {
-        :id => orig.id
-      }
+    get(:copy, :params => {:id => orig.id})
     assert_response :success
 
     assert_select 'textarea[name=?]', 'project[description]', :text => orig.description
@@ -1095,9 +1208,7 @@ class ProjectsControllerTest < Redmine::ControllerTest
 
   def test_get_copy_with_invalid_source_should_respond_with_404
     @request.session[:user_id] = 1
-    get :copy, :params => {
-        :id => 99
-      }
+    get(:copy, :params => {:id => 99})
     assert_response 404
   end
 
@@ -1107,9 +1218,7 @@ class ProjectsControllerTest < Redmine::ControllerTest
     source = Project.generate!(:issue_custom_fields => [field1])
     @request.session[:user_id] = 1
 
-    get :copy, :params => {
-        :id => source.id
-      }
+    get(:copy, :params => {:id => source.id})
     assert_response :success
     assert_select 'input[type=hidden][name=?][value=?]', 'project[issue_custom_field_ids][]', field1.id.to_s
     assert_select 'input[type=hidden][name=?][value=?]', 'project[issue_custom_field_ids][]', field2.id.to_s, 0
@@ -1120,17 +1229,19 @@ class ProjectsControllerTest < Redmine::ControllerTest
     CustomField.delete_all
 
     assert_difference 'Project.count' do
-      post :copy, :params => {
+      post(
+        :copy,
+        :params => {
           :id => 1,
           :project => {
             :name => 'Copy',
             :identifier => 'unique-copy',
             :tracker_ids => ['1', '2', '3', ''],
             :enabled_module_names => %w(issue_tracking time_tracking)
-
           },
           :only => %w(issues versions)
         }
+      )
     end
     project = Project.find('unique-copy')
     source = Project.find(1)
@@ -1143,33 +1254,39 @@ class ProjectsControllerTest < Redmine::ControllerTest
 
   def test_post_copy_should_redirect_to_settings_when_successful
     @request.session[:user_id] = 1 # admin
-    post :copy, :params => {
+    post(
+      :copy,
+      :params => {
         :id => 1,
         :project => {
           :name => 'Copy',
           :identifier => 'unique-copy'
         }
       }
+    )
     assert_response :redirect
     assert_redirected_to :controller => 'projects', :action => 'settings', :id => 'unique-copy'
   end
 
   def test_post_copy_with_failure
     @request.session[:user_id] = 1
-    post :copy, :params => {
+    post(
+      :copy,
+      :params => {
         :id => 1,
         :project => {
           :name => 'Copy',
           :identifier => ''
         }
       }
+    )
     assert_response :success
     assert_select_error /Identifier cannot be blank/
   end
 
   def test_bookmark_should_create_bookmark
     @request.session[:user_id] = 3
-    post :bookmark, params: { id: 'ecookbook' }
+    post(:bookmark, :params => {:id => 'ecookbook'})
     assert_redirected_to controller: 'projects', action: 'show', id: 'ecookbook'
     jb = Redmine::ProjectJumpBox.new(User.find(3))
     assert jb.bookmark?(Project.find('ecookbook'))
@@ -1181,7 +1298,7 @@ class ProjectsControllerTest < Redmine::ControllerTest
     jb = Redmine::ProjectJumpBox.new(User.find(3))
     project = Project.find('ecookbook')
     jb.bookmark_project project
-    delete :bookmark, params: { id: 'ecookbook' }
+    delete(:bookmark, :params => {:id => 'ecookbook'})
     assert_redirected_to controller: 'projects', action: 'show', id: 'ecookbook'
 
     jb = Redmine::ProjectJumpBox.new(User.find(3))
@@ -1189,47 +1306,54 @@ class ProjectsControllerTest < Redmine::ControllerTest
   end
 
   def test_index_jump_without_project_id_should_redirect_to_active_tab
-    get :index, :params => {
-        :jump => 'issues'
-      }
+    get(:index, :params => {:jump => 'issues'})
     assert_redirected_to '/issues'
   end
 
   def test_index_jump_should_not_redirect_to_unknown_tab
-    get :index, :params => {
+    get(
+      :index,
+      :params => {
         :jump => 'foobar'
       }
+    )
     assert_response :success
   end
 
   def test_show_jump_should_redirect_to_active_tab
-    get :show, :params => {
+    get(
+      :show,
+      :params => {
         :id => 1,
         :jump => 'issues'
       }
+    )
     assert_redirected_to '/projects/ecookbook/issues'
   end
 
   def test_show_jump_should_not_redirect_to_inactive_tab
-    get :show, :params => {
+    get(
+      :show, :params => {
         :id => 3,
         :jump => 'documents'
       }
+    )
     assert_response :success
   end
 
   def test_show_jump_should_not_redirect_to_unknown_tab
-    get :show, :params => {
+    get(
+      :show,
+      :params => {
         :id => 3,
         :jump => 'foobar'
       }
+    )
     assert_response :success
   end
 
   def test_body_should_have_project_css_class
-    get :show, :params => {
-        :id => 1
-      }
+    get(:show, :params => {:id => 1})
     assert_select 'body.project-ecookbook'
   end
 
