@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,8 +24,15 @@ class Redmine::NotifiableTest < ActiveSupport::TestCase
   end
 
   def test_all
-    %w(issue_added issue_updated issue_note_added issue_status_updated issue_assigned_to_updated issue_priority_updated news_added news_comment_added document_added file_added message_posted wiki_content_added wiki_content_updated).each do |notifiable|
-      assert Redmine::Notifiable.all.collect(&:name).include?(notifiable), "missing #{notifiable}"
+    %w(issue_added issue_updated issue_note_added issue_status_updated
+       issue_assigned_to_updated issue_priority_updated news_added
+       news_comment_added document_added file_added
+       message_posted
+       wiki_content_added wiki_content_updated).each do |notifiable|
+      assert(
+        Redmine::Notifiable.all.collect(&:name).include?(notifiable),
+        "missing #{notifiable}"
+      )
     end
   end
 end
