@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -25,7 +27,6 @@ class MercurialAdapterTest < ActiveSupport::TestCase
   HgCommandArgumentError = Redmine::Scm::Adapters::MercurialAdapter::HgCommandArgumentError
 
   REPOSITORY_PATH = repository_path('mercurial')
-  CHAR_1_HEX = "\xc3\x9c"
 
   if File.directory?(REPOSITORY_PATH)
     def setup
@@ -35,29 +36,32 @@ class MercurialAdapterTest < ActiveSupport::TestCase
       assert_equal true, adapter_class.client_available
       assert_equal true, adapter_class.client_version_above?([0, 9, 5])
 
-      @adapter = Redmine::Scm::Adapters::MercurialAdapter.new(
-                            REPOSITORY_PATH,
-                            nil,
-                            nil,
-                            nil,
-                           'ISO-8859-1')
+      @adapter =
+        Redmine::Scm::Adapters::MercurialAdapter.new(
+          REPOSITORY_PATH,
+          nil,
+          nil,
+          nil,
+          'ISO-8859-1'
+        )
       @diff_c_support = true
-      @char_1        = CHAR_1_HEX.dup.force_encoding('UTF-8')
-      @tag_char_1    = "tag-#{CHAR_1_HEX}-00".force_encoding('UTF-8')
-      @branch_char_0 = "branch-#{CHAR_1_HEX}-00".force_encoding('UTF-8')
-      @branch_char_1 = "branch-#{CHAR_1_HEX}-01".force_encoding('UTF-8')
+      @char_1        = 'Ü'
+      @tag_char_1    = 'tag-Ü-00'
+      @branch_char_0 = 'branch-Ü-00'
+      @branch_char_1 = 'branch-Ü-01'
     end
 
     def test_hgversion
-      to_test = { "Mercurial Distributed SCM (version 0.9.5)\n"  => [0,9,5],
-                  "Mercurial Distributed SCM (1.0)\n"            => [1,0],
-                  "Mercurial Distributed SCM (1e4ddc9ac9f7+20080325)\n" => nil,
-                  "Mercurial Distributed SCM (1.0.1+20080525)\n" => [1,0,1],
-                  "Mercurial Distributed SCM (1916e629a29d)\n"   => nil,
-                  "Mercurial SCM Distribuito (versione 0.9.5)\n" => [0,9,5],
-                  "(1.6)\n(1.7)\n(1.8)"                          => [1,6],
-                  "(1.7.1)\r\n(1.8.1)\r\n(1.9.1)"                => [1,7,1]}
-
+      to_test = {
+        "Mercurial Distributed SCM (version 0.9.5)\n"  => [0, 9, 5],
+        "Mercurial Distributed SCM (1.0)\n"            => [1, 0],
+        "Mercurial Distributed SCM (1e4ddc9ac9f7+20080325)\n" => nil,
+        "Mercurial Distributed SCM (1.0.1+20080525)\n" => [1, 0, 1],
+        "Mercurial Distributed SCM (1916e629a29d)\n"   => nil,
+        "Mercurial SCM Distribuito (versione 0.9.5)\n" => [0, 9, 5],
+        "(1.6)\n(1.7)\n(1.8)"                          => [1, 6],
+        "(1.7.1)\r\n(1.8.1)\r\n(1.9.1)"                => [1, 7, 1]
+      }
       to_test.each do |s, v|
         test_hgversion_for(s, v)
       end
@@ -65,13 +69,13 @@ class MercurialAdapterTest < ActiveSupport::TestCase
 
     def test_template_path
       to_test = {
-                  [1,2]    => "1.0",
-                  []       => "1.0",
-                  [1,2,1]  => "1.0",
-                  [1,7]    => "1.0",
-                  [1,7,1]  => "1.0",
-                  [2,0]    => "1.0",
-                 }
+        [1, 2]    => "1.0",
+        []        => "1.0",
+        [1, 2, 1] => "1.0",
+        [1, 7]    => "1.0",
+        [1, 7, 1] => "1.0",
+        [2, 0]    => "1.0",
+      }
       to_test.each do |v, template|
         test_template_path_for(v, template)
       end
@@ -79,12 +83,12 @@ class MercurialAdapterTest < ActiveSupport::TestCase
 
     def test_info
       [REPOSITORY_PATH, REPOSITORY_PATH + "/",
-           REPOSITORY_PATH + "//"].each do |repo|
+       REPOSITORY_PATH + "//"].each do |repo|
         adp = Redmine::Scm::Adapters::MercurialAdapter.new(repo)
-        repo_path =  adp.info.root_url.gsub(/\\/, "/")
+        repo_path =  adp.info.root_url.tr('\\', "/")
         assert_equal REPOSITORY_PATH, repo_path
-        assert_equal '33', adp.info.lastrev.revision
-        assert_equal '2e6d546429230f377d7d19c2078abd2dd909f235',adp.info.lastrev.scmid
+        assert_equal '39', adp.info.lastrev.revision
+        assert_equal '04aed9840e9266e535f5f20f7e42c9f9f84f9cf4', adp.info.lastrev.scmid
       end
     end
 
@@ -102,6 +106,13 @@ class MercurialAdapterTest < ActiveSupport::TestCase
       assert_equal '400bb86721098697c7d17b3724c794c57636de70', revisions[0].scmid
     end
 
+    def test_ctrl_character
+      revisions = @adapter.revisions(nil, '619ebf674dab', '619ebf674dab')
+      assert_equal 1, revisions.size
+      assert_equal "ctrl-s\u0013user", revisions[0].author
+      assert_equal "ctrl-s\u0013message", revisions[0].message
+    end
+
     def test_parents
       revs1 = @adapter.revisions(nil, 0, 0)
       assert_equal 1, revs1.size
@@ -122,7 +133,7 @@ class MercurialAdapterTest < ActiveSupport::TestCase
         assert_nil @adapter.diff(nil, '100000')
       end
       assert_nil @adapter.diff(nil, '100000', '200000')
-      [2, '400bb8672109', '400', 400].each do |r1|
+      [2, '400bb8672109', '400', 400].freeze.each do |r1|
         diff1 = @adapter.diff(nil, r1)
         if @diff_c_support
           assert_equal 28, diff1.size
@@ -131,7 +142,7 @@ class MercurialAdapterTest < ActiveSupport::TestCase
         else
           assert_equal 0, diff1.size
         end
-        [4, 'def6d2f1254a'].each do |r2|
+        [4, 'def6d2f1254a'].freeze.each do |r2|
           diff2 = @adapter.diff(nil, r1, r2)
           assert_equal 49, diff2.size
           buf =  diff2[41].gsub(/\r\n|\r|\n/, "")
@@ -155,7 +166,7 @@ class MercurialAdapterTest < ActiveSupport::TestCase
 
     def test_diff_made_by_revision
       if @diff_c_support
-        [24, '24', '4cddb4e45f52'].each do |r1|
+        [24, '24', '4cddb4e45f52'].freeze.each do |r1|
           diff1 = @adapter.diff(nil, r1)
           assert_equal 5, diff1.size
           buf = diff1[4].gsub(/\r\n|\r|\n/, "")
@@ -165,7 +176,7 @@ class MercurialAdapterTest < ActiveSupport::TestCase
     end
 
     def test_cat
-      [2, '400bb8672109', '400', 400].each do |r|
+      [2, '400bb8672109', '400', 400].freeze.each do |r|
         buf = @adapter.cat('sources/welcome_controller.rb', r)
         assert buf
         lines = buf.split("\r\n")
@@ -177,7 +188,7 @@ class MercurialAdapterTest < ActiveSupport::TestCase
 
     def test_annotate
       assert_equal [], @adapter.annotate("sources/welcome_controller.rb").lines
-      [2, '400bb8672109', '400', 400].each do |r|
+      [2, '400bb8672109', '400', 400].freeze.each do |r|
         ann = @adapter.annotate('sources/welcome_controller.rb', r)
         assert ann
         assert_equal '1', ann.revisions[17].revision
@@ -194,7 +205,7 @@ class MercurialAdapterTest < ActiveSupport::TestCase
       assert_equal 1, @adapter.entries("sources", 3).size
       assert_equal 1, @adapter.entries("sources", 'b3a615152df8').size
 
-      [2, '400bb8672109', '400', 400].each do |r|
+      [2, '400bb8672109', '400', 400].freeze.each do |r|
         entries1 = @adapter.entries(nil, r)
         assert entries1
         assert_equal 3, entries1.size
@@ -273,8 +284,8 @@ class MercurialAdapterTest < ActiveSupport::TestCase
       assert_nil @adapter.entry('invalid/invalid/')
       assert_nil @adapter.entry('/invalid/invalid')
       assert_nil @adapter.entry('/invalid/invalid/')
-      ["README", "/README"].each do |path|
-        ["0", "0885933ad4f6", "0885933ad4f68d77c2649cd11f8311276e7ef7ce"].each do |rev|
+      ["README", "/README"].freeze.each do |path|
+        ["0", "0885933ad4f6", "0885933ad4f68d77c2649cd11f8311276e7ef7ce"].freeze.each do |rev|
           entry = @adapter.entry(path, rev)
           assert_equal "README", entry.path
           assert_equal "file", entry.kind
@@ -282,15 +293,15 @@ class MercurialAdapterTest < ActiveSupport::TestCase
           assert_equal '0885933ad4f68d77c2649cd11f8311276e7ef7ce', entry.lastrev.identifier
         end
       end
-      ["sources", "/sources", "/sources/"].each do |path|
-        ["0", "0885933ad4f6", "0885933ad4f68d77c2649cd11f8311276e7ef7ce"].each do |rev|
+      ["sources", "/sources", "/sources/"].freeze.each do |path|
+        ["0", "0885933ad4f6", "0885933ad4f68d77c2649cd11f8311276e7ef7ce"].freeze.each do |rev|
           entry = @adapter.entry(path, rev)
           assert_equal "sources", entry.path
           assert_equal "dir", entry.kind
         end
       end
-      ["sources/watchers_controller.rb", "/sources/watchers_controller.rb"].each do |path|
-        ["0", "0885933ad4f6", "0885933ad4f68d77c2649cd11f8311276e7ef7ce"].each do |rev|
+      ["sources/watchers_controller.rb", "/sources/watchers_controller.rb"].freeze.each do |path|
+        ["0", "0885933ad4f6", "0885933ad4f68d77c2649cd11f8311276e7ef7ce"].freeze.each do |rev|
           entry = @adapter.entry(path, rev)
           assert_equal "sources/watchers_controller.rb", entry.path
           assert_equal "file", entry.kind
@@ -318,67 +329,97 @@ class MercurialAdapterTest < ActiveSupport::TestCase
     end
 
     def test_tags
-      assert_equal [@tag_char_1, 'tag_test.00', 'tag-init-revision'], @adapter.tags
+      tags = ['double"quote"tag', @tag_char_1, 'tag_test.00', 'tag-init-revision']
+      assert_equal tags, @adapter.tags
     end
 
     def test_tagmap
-      tm = {
-        @tag_char_1         => 'adf805632193500ad3b615cd04f58f9b0769f576',
-        'tag_test.00'       => '6987191f453a5f6557018d522feea2c450d5588d',
-        'tag-init-revision' => '0885933ad4f68d77c2649cd11f8311276e7ef7ce',
+      tm =
+        {
+          'double"quote"tag'  => 'cf5f7c556f5a643e1ec7cb01775be539f64eeefb',
+          @tag_char_1         => 'adf805632193500ad3b615cd04f58f9b0769f576',
+          'tag_test.00'       => '6987191f453a5f6557018d522feea2c450d5588d',
+          'tag-init-revision' => '0885933ad4f68d77c2649cd11f8311276e7ef7ce',
         }
       assert_equal tm, @adapter.tagmap
     end
 
     def test_branches
-      brs = []
+      branches = []
       @adapter.branches.each do |b|
-        brs << b
+        branches << b
       end
-      assert_equal 7, brs.length
-      assert_equal 'default', brs[0].to_s
-      assert_equal '31', brs[0].revision
-      assert_equal '31eeee7395c8c78e66dd54c50addd078d10b2355', brs[0].scmid
-      assert_equal 'test-branch-01', brs[1].to_s
-      assert_equal '30', brs[1].revision
-      assert_equal 'ad4dc4f80284a4f9168b77e0b6de288e5d207ee7', brs[1].scmid
-      assert_equal @branch_char_1, brs[2].to_s
-      assert_equal '27', brs[2].revision
-      assert_equal '7bbf4c738e7145149d2e5eb1eed1d3a8ddd3b914', brs[2].scmid
-      assert_equal 'branch (1)[2]&,%.-3_4', brs[3].to_s
-      assert_equal '25', brs[3].revision
-      assert_equal 'afc61e85bde74de930e5846c8451bd55b5bafc9c', brs[3].scmid
-      assert_equal @branch_char_0, brs[4].to_s
-      assert_equal '23', brs[4].revision
-      assert_equal 'c8d3e4887474af6a589190140508037ebaa9d9c3', brs[4].scmid
-      assert_equal 'test_branch.latin-1', brs[5].to_s
-      assert_equal '22', brs[5].revision
-      assert_equal 'c2ffe7da686aa3d956e59f2a2854cf8980a8b768', brs[5].scmid
-      assert_equal 'test-branch-00', brs[6].to_s
-      assert_equal '13', brs[6].revision
-      assert_equal '3a330eb329586ea2adb3f83237c23310e744ebe9', brs[6].scmid
+      assert_equal 9, branches.length
+
+      branch = branches[-9]
+      assert_equal 'double"quote"branch', branch.to_s
+      assert_equal '39', branch.revision
+      assert_equal '04aed9840e9266e535f5f20f7e42c9f9f84f9cf4', branch.scmid
+
+      branch = branches[-8]
+      assert_equal 'issue-23055-ctrl-char', branch.to_s
+      assert_equal '35', branch.revision
+      assert_equal '3e998343166a1b8273973bcd46dd2bad74344d74', branch.scmid
+
+      branch = branches[-7]
+      assert_equal 'default', branch.to_s
+      assert_equal '31', branch.revision
+      assert_equal '31eeee7395c8c78e66dd54c50addd078d10b2355', branch.scmid
+
+      branch = branches[-6]
+      assert_equal 'test-branch-01', branch.to_s
+      assert_equal '30', branch.revision
+      assert_equal 'ad4dc4f80284a4f9168b77e0b6de288e5d207ee7', branch.scmid
+
+      branch = branches[-5]
+      assert_equal @branch_char_1, branch.to_s
+      assert_equal '27', branch.revision
+      assert_equal '7bbf4c738e7145149d2e5eb1eed1d3a8ddd3b914', branch.scmid
+
+      branch = branches[-4]
+      assert_equal 'branch (1)[2]&,%.-3_4', branch.to_s
+      assert_equal '25', branch.revision
+      assert_equal 'afc61e85bde74de930e5846c8451bd55b5bafc9c', branch.scmid
+
+      branch = branches[-3]
+      assert_equal @branch_char_0, branch.to_s
+      assert_equal '23', branch.revision
+      assert_equal 'c8d3e4887474af6a589190140508037ebaa9d9c3', branch.scmid
+
+      branch = branches[-2]
+      assert_equal 'test_branch.latin-1', branch.to_s
+      assert_equal '22', branch.revision
+      assert_equal 'c2ffe7da686aa3d956e59f2a2854cf8980a8b768', branch.scmid
+
+      branch = branches[-1]
+      assert_equal 'test-branch-00', branch.to_s
+      assert_equal '13', branch.revision
+      assert_equal '3a330eb329586ea2adb3f83237c23310e744ebe9', branch.scmid
     end
 
     def test_branchmap
-      bm = {
-         'default'               => '31eeee7395c8c78e66dd54c50addd078d10b2355',
-         'test_branch.latin-1'   => 'c2ffe7da686aa3d956e59f2a2854cf8980a8b768',
-         'branch (1)[2]&,%.-3_4' => 'afc61e85bde74de930e5846c8451bd55b5bafc9c',
-         'test-branch-00'        => '3a330eb329586ea2adb3f83237c23310e744ebe9',
-         "test-branch-01"        => 'ad4dc4f80284a4f9168b77e0b6de288e5d207ee7',
-         @branch_char_0          => 'c8d3e4887474af6a589190140508037ebaa9d9c3',
-         @branch_char_1          => '7bbf4c738e7145149d2e5eb1eed1d3a8ddd3b914',
-       }
+      bm =
+        {
+          'double"quote"branch'   => '04aed9840e9266e535f5f20f7e42c9f9f84f9cf4',
+          'issue-23055-ctrl-char' => '3e998343166a1b8273973bcd46dd2bad74344d74',
+          'default'               => '31eeee7395c8c78e66dd54c50addd078d10b2355',
+          'test_branch.latin-1'   => 'c2ffe7da686aa3d956e59f2a2854cf8980a8b768',
+          'branch (1)[2]&,%.-3_4' => 'afc61e85bde74de930e5846c8451bd55b5bafc9c',
+          'test-branch-00'        => '3a330eb329586ea2adb3f83237c23310e744ebe9',
+          "test-branch-01"        => 'ad4dc4f80284a4f9168b77e0b6de288e5d207ee7',
+          @branch_char_0          => 'c8d3e4887474af6a589190140508037ebaa9d9c3',
+          @branch_char_1          => '7bbf4c738e7145149d2e5eb1eed1d3a8ddd3b914',
+        }
       assert_equal bm, @adapter.branchmap
     end
 
     def test_path_space
       p = 'README (1)[2]&,%.-3_4'
-      [15, '933ca60293d7'].each do |r1|
+      [15, '933ca60293d7'].freeze.each do |r1|
         assert @adapter.diff(p, r1)
         assert @adapter.cat(p, r1)
         assert_equal 1, @adapter.annotate(p, r1).lines.length
-        [25, 'afc61e85bde7'].each do |r2|
+        [25, 'afc61e85bde7'].freeze.each do |r2|
           assert @adapter.diff(p, r1, r2)
         end
       end
@@ -397,51 +438,54 @@ class MercurialAdapterTest < ActiveSupport::TestCase
     end
 
     def test_nodes_in_branch
-       [
-          'default',
-          @branch_char_1,
-          'branch (1)[2]&,%.-3_4',
-          @branch_char_0,
-          'test_branch.latin-1',
-          'test-branch-00',
-             ].each do |bra|
-        nib0 = @adapter.nodes_in_branch(bra)
+      [
+        'default',
+        @branch_char_1,
+        'branch (1)[2]&,%.-3_4',
+        @branch_char_0,
+        'test_branch.latin-1',
+        'test-branch-00',
+      ].freeze.
+      each do |branch|
+        nib0 = @adapter.nodes_in_branch(branch)
         assert nib0
-        nib1 = @adapter.nodes_in_branch(bra, :limit => 1)
+        nib1 = @adapter.nodes_in_branch(branch, :limit => 1)
         assert_equal 1, nib1.size
-        case bra
-          when 'branch (1)[2]&,%.-3_4'
-            if @adapter.class.client_version_above?([1, 6])
-              assert_equal 3, nib0.size
-              assert_equal 'afc61e85bde74de930e5846c8451bd55b5bafc9c', nib0[0]
-              nib2 = @adapter.nodes_in_branch(bra, :limit => 2)
-              assert_equal 2, nib2.size
-              assert_equal '933ca60293d78f7c7979dd123cc0c02431683575', nib2[1]
-            end
-          when @branch_char_1
-            if @adapter.class.client_version_above?([1, 6])
-              assert_equal 2, nib0.size
-              assert_equal '08ff3227303ec0dfcc818efa8e9cc652fe81859f', nib0[1]
-              nib2 = @adapter.nodes_in_branch(bra, :limit => 1)
-              assert_equal 1, nib2.size
-              assert_equal '7bbf4c738e7145149d2e5eb1eed1d3a8ddd3b914', nib2[0]
-            end
+        case branch
+        when 'branch (1)[2]&,%.-3_4'
+          if @adapter.class.client_version_above?([1, 6])
+            assert_equal 3, nib0.size
+            assert_equal 'afc61e85bde74de930e5846c8451bd55b5bafc9c', nib0[0]
+            nib2 = @adapter.nodes_in_branch(branch, :limit => 2)
+            assert_equal 2, nib2.size
+            assert_equal '933ca60293d78f7c7979dd123cc0c02431683575', nib2[1]
+          end
+        when @branch_char_1
+          if @adapter.class.client_version_above?([1, 6])
+            assert_equal 2, nib0.size
+            assert_equal '08ff3227303ec0dfcc818efa8e9cc652fe81859f', nib0[1]
+            nib2 = @adapter.nodes_in_branch(branch, :limit => 1)
+            assert_equal 1, nib2.size
+            assert_equal '7bbf4c738e7145149d2e5eb1eed1d3a8ddd3b914', nib2[0]
+          end
         end
       end
     end
 
     def test_path_encoding_default_utf8
-      adpt1 = Redmine::Scm::Adapters::MercurialAdapter.new(
-                                REPOSITORY_PATH
-                              )
+      adpt1 =
+        Redmine::Scm::Adapters::MercurialAdapter.new(
+          REPOSITORY_PATH
+        )
       assert_equal "UTF-8", adpt1.path_encoding
-      adpt2 = Redmine::Scm::Adapters::MercurialAdapter.new(
-                                REPOSITORY_PATH,
-                                nil,
-                                nil,
-                                nil,
-                                ""
-                              )
+      adpt2 =
+        Redmine::Scm::Adapters::MercurialAdapter.new(
+          REPOSITORY_PATH,
+          nil,
+          nil,
+          nil,
+          ""
+        )
       assert_equal "UTF-8", adpt2.path_encoding
     end
 
