@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -102,8 +102,9 @@ class TimeEntryActivityTest < ActiveSupport::TestCase
   def test_system_activity_with_child_in_use_should_be_in_use
     project = Project.generate!
     system_activity = TimeEntryActivity.create!(:name => 'Activity')
-    project_activity = TimeEntryActivity.create!(:name => 'Activity', :project => project, :parent_id => system_activity.id)
-
+    project_activity =
+      TimeEntryActivity.create!(:name => 'Activity', :project => project,
+                                :parent_id => system_activity.id)
     TimeEntry.generate!(:project => project, :activity => project_activity)
 
     assert project_activity.in_use?
@@ -115,7 +116,9 @@ class TimeEntryActivityTest < ActiveSupport::TestCase
     entries = []
     system_activity = TimeEntryActivity.create!(:name => 'Activity')
     entries << TimeEntry.generate!(:project => project, :activity => system_activity)
-    project_activity = TimeEntryActivity.create!(:name => 'Activity', :project => project, :parent_id => system_activity.id)
+    project_activity =
+      TimeEntryActivity.create!(:name => 'Activity', :project => project,
+                                :parent_id => system_activity.id)
     entries << TimeEntry.generate!(:project => project.reload, :activity => project_activity)
     assert_difference 'TimeEntryActivity.count', -2 do
       assert_nothing_raised do
@@ -131,4 +134,40 @@ class TimeEntryActivityTest < ActiveSupport::TestCase
     assert_include activity, project.activities
     assert_include TimeEntryActivity.find(9), project.activities
   end
+
+  def test_project_activity_should_have_the_same_position_as_parent_activity
+    project = Project.find(1)
+
+    parent_activity = TimeEntryActivity.find_by(position: 3, parent_id: nil)
+    project.update_or_create_time_entry_activities(
+      {
+        parent_activity.id.to_s => {
+          'parent_id' => parent_activity.id.to_s,
+          'active' => '0',
+          'custom_field_values' => {'7' => ''}
+        }
+      }
+    )
+    project_activity = TimeEntryActivity.find_by(position: 3, parent_id: parent_activity.id, project_id: 1)
+    assert_equal parent_activity.position, project_activity.position
+
+    # Changing the position of the parent activity also changes the position of the activity in each project.
+    other_parent_activity = TimeEntryActivity.find_by(position: 4, parent_id: nil)
+    project.update_or_create_time_entry_activities(
+      {
+        other_parent_activity.id.to_s => {
+          'parent_id' => other_parent_activity.id.to_s,
+          'active' => '0',
+          'custom_field_values' => {'7' => ''}
+        }
+      }
+    )
+    other_project_activity = TimeEntryActivity.find_by(position: 4, parent_id: other_parent_activity.id, project_id: 1)
+
+    parent_activity.update(position: 4)
+    assert_equal 4, parent_activity.reload.position
+    assert_equal parent_activity.position, project_activity.reload.position
+    assert_equal 3, other_parent_activity.reload.position
+    assert_equal other_parent_activity.position, other_project_activity.reload.position
+  end
 end
