@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -50,6 +50,13 @@ class UserPreferenceTest < ActiveSupport::TestCase
     assert_equal true, preference.no_self_notified
   end
 
+  def test_no_self_notified_should_default_to_setting
+    with_settings :default_users_no_self_notified => '0' do
+      preference = UserPreference.new
+      assert_equal false, preference.no_self_notified
+    end
+  end
+
   def test_create
     user = User.new(:firstname => "new", :lastname => "user", :mail => "newuser@somenet.foo")
     user.login = "newuser"
@@ -110,4 +117,11 @@ class UserPreferenceTest < ActiveSupport::TestCase
     up.save!
     assert_equal ['documents'], up.my_page_settings.keys
   end
+
+  def test_toolbar_language_options_setter_should_remove_except_supported_languages
+    up = User.find(2).pref
+    # bar is not a supported language
+    up.toolbar_language_options = 'ruby,cpp,bar,c'
+    assert_equal 'ruby,cpp,c', up.toolbar_language_options
+  end
 end
